/*
 *  This sample code is public domain. Use it however you wish at your own risk. 
 */ 

package org.example.georg.connector.gc;

import android.content.ContentProvider;
import android.content.ContentValues;
import android.database.Cursor;
import android.database.MatrixCursor;
import android.net.Uri;
import android.util.Log;

/**
 * This is a sample content-provider for use with GeOrg. Since GeOrg only want's to
 * gather information, only the query-method is implemented. 
 * By looking at the given URL it determines which concrete implementation to call. 
 * To write a full connector you have to answer to each of these URLs:
 * 
 * content://<your package>/cacheByCoord
 * content://<your package>/cacheDetails
 * content://<your package>/readGPX
 * content://<your package>/ping
 * 
 * These are explained in the query-method. There are more queries you might want to implement, though.
 * The full list of supported Queries is listed in the ConnectorSupportedQueries-Class. 
 *
 */
public class GCHttpConnector extends ContentProvider {
	
	private final static String TAG = "GCHttpConnector";
	private GCLoader ivLoader;
	
	/**
	 * creating the connector and initializing the concrete implementation
	 * @see android.content.ContentProvider#onCreate()
	 */
	@Override
	public boolean onCreate() {
		Log.v(TAG, "onCreate()");
		ivLoader = new GCLoader(getContext());
		return true;
	}
	
	/**
	 * Tries to gather some data. What to do exactly is decided by parsing the content://-URI 
	 * for the action-suffix. The following actions must be supported:
	 * 
	 * <ul>
	 * 
	 *  <li><strong>content://&lt;your package&gt;/cacheByCoord</strong><br><br>
	 *  Read a list of caches within a coordinate rectangle. Returns a Cursor to the result.<br><br>
	 *  <strong>input-parameters:</strong><br>
	 *  <ol>
	 *  	<li>latitude of northmost cache to find, as int (e.g. 52.234561 would be given as 52234561)</li>
	 *  	<li>longitude of westmost cache to find, as int</li>
	 *  	<li>latitude of southmost cache to find, as int</li>
	 *  	<li>longitude of eastmost cache to find, as int</li>
	 *  	<li>a filter to apply for the result. This is a &-concatenated String with filterwords.
	 *          Currently GeOrg will send this filter:
	 *        <ul>
	 *           <li>found=[fdn]<br>The parameter is a concatenated set of characters 
	 *           indicating what may be shown on the map. f=found caches,
	 *            d=dnf caches, n=not yet found caches 
	 *           </li>
	 *        </ul>
	 *      </li>
	 *  	<li>an indicator whether to return add.wps (true/false). optional. default: false</li>
	 *  </ol>
	 *  <strong>result-cursor-rows:</strong><br>
	 *  	<ul>
	 * 			<li>"gcid"    - GC code</li>
	 * 			<li>"name"    - cachename</li>
	 * 			<li>"type"    - cachetype</li>
	 * 			<li>"lat"     - latitude of cache</li>
	 * 			<li>"lon"     - longiture of cache</li>
	 * 			<li>"id"      - later used when getting the cache-details</li>
	 * 			<li>"active"  - is cache active?</li>
	 * 			<li>"found"   - has user found cache?</li>
	 * 			<li>"message" - error-message</li>
	 *  	</ul>
	 *  </li>
	 *  
	 *  <li><strong>content://&lt;your package&gt;/cacheDetails</strong><br><br>
	 *  Reads details for a cache displayed on the map. Needs the ID from the cache gotten via
	 *  the readCachesByCoord-action.<br><br>
	 *  <strong>input-parameters:</strong><br>
	 *  <ol>
	 *  	<li>the id of the cache from the readCachesByCoord-action</li>
	 *  </ol>
	 *  <strong>result-cursor-columns:</strong><br>
	 *  	<ul>
	 * 		<li>"owner"    - owner of the cache</li>
	 * 		<li>"diff"     - diff of cache</li>
	 * 		<li>"terr"     - terrain of cache</li>
	 * 		<li>"url"      - url for online viewing</li>
	 * 		<li>"size"     - size of the cachecontainer</li>
	 * 		<li>"message"   - errormessage</li>
	 *  	</ul>
	 *  </li>
	 *  
	 *  <li><strong>content://&lt;your package&gt;/readGPX</strong><br><br>
	 *  Read a GPX-file from the Server for a given GCID. Returns a Cursor to the Result. 
	 * 	The resulting table should have only one row containing the gpx-data as string in the
	 * 	column "gpx".<br><br>
	 *  <strong>input-parameters:</strong><br>
	 *  <ol>
	 *  	<li>the gc code of the cache to download</li>
	 *  	<li>an optional directory to store spoilers in. may be null</li>
	 *  </ol>
	 *  <strong>result-cursor-columns:</strong><br>
	 *  	<ul>
	 * 		<li>"gpx"       - the GPX-data as string</li>
	 * 		<li>"message"   - errormessage</li>
	 *  	</ul>
	 *  </li>
	 *  
	 *  <li><strong>content://&lt;your package&gt;/readSpoilers</strong><br><br>
	 *  Downloads spoilers for a given cache.<br><br>
	 *  <strong>input-parameters:</strong><br>
	 *  <ol>
	 *  	<li>the gc code of the cache to download spoilers for</li>
	 *  	<li>an directory to store spoilers in</li>
	 *  </ol>
	 *  <strong>result-cursor-columns:</strong><br>
	 *  	<ul>
	 * 		<li>"number"    - number of spoilers downloaded</li>
	 * 		<li>"message"   - errormessage</li>
	 *  	</ul>
	 *  </li>
	 *  
	 *  <li><strong>content://&lt;your package&gt;/ping</strong><br><br>
	 *  Technical call to see if the connector is alive. This is used by GeOrg upon registration
	 *  to check the validity of the connector's URI. This might also be called at any later 
	 *  time for the same purpose.<br><br>
	 *  <strong>input-parameters: </strong>none<br><br>
	 *  <strong>result-cursor-columns:</strong><br>
	 *  	<ul>
	 * 		<li>"result"    - one row with the String "OK" in it.</li>
	 *  	</ul>
	 *  </li>
	 *  
	 *  <li><strong>content://&lt;your package&gt;/supportedQueries</strong><br><br>
	 *  List all queries that are supported by this connector.
	 *  <strong>input-parameters: </strong>none<br><br>
	 *  <strong>result-cursor-columns:</strong><br>
	 *  	<ul>
	 * 		<li>"query"    - one row with the url-suffix of each supported Query. 
	 *           Aforementioned queries in this javadoc are mandatory.</li>
	 *  	</ul>
	 *  </li>
	 *  
	 *  <li><strong>content://&lt;your package&gt;/writeLog</strong><br><br>
	 *  Writes a Cachelog to the backend.
	 *  <strong>input-parameters: </strong>
	 *   <ol>
	 *  	<li>the gc code of the cache to log</li>
	 *  	<li>the type of the log as String</li>
	 *  	<li>the text of the log</li>
	 *  	<li>the day of the log</li>
	 *  	<li>the month of the log</li>
	 *  	<li>the year of the log</li>
	 *  </ol>
	 *  <strong>result-cursor-columns:</strong><br>
	 *  	<ul>
	 * 		<li>"msg"    - "ok" if all went well - else: an error-message.</li>
	 *  	</ul>
	 *  </li>
	 *  
	 *  <li><strong>content://&lt;your package&gt;/pushFieldNotes</strong><br><br>
	 *  Writes a fieldnotes-file in Groundspeak-format to the backend.
	 *  <strong>input-parameters: </strong>
	 *   <ol>
	 *  	<li>a filename of a file on the handset containing the fieldnotes</li>
	 *  </ol>
	 *  <strong>result-cursor-columns:</strong><br>
	 *  	<ul>
	 * 		<li>"msg"    - "ok" if all went well - else: an error-message.</li>
	 *  	</ul>
	 *  </li>
	 *  
	 *  <li><strong>content://&lt;your package&gt;/getCommands</strong><br><br>
	 *  Gets all commands this connector understands to invoke them directly in GeOrg.
	 *  <strong>input-parameters: </strong>none<br><br>
	 *  <strong>result-cursor-columns:</strong><br>
	 *  	<ul>
	 * 		<li>"name"     - name of each command.</li>
	 * 		<li>"readable" - user-readable/descriptive name of the command to display as an option</li>
	 *  	</ul>
	 *  </li>
	 *  
	 *  <li><strong>content://&lt;your package&gt;/executeCommand</strong><br><br>
	 *  Executes a command from getCommands list of commands.
	 *  <strong>input-parameters: </strong>
	 *   <ol>
	 *  	<li>The name of the command as returned by getCommands</li>
	 *  </ol>
	 *  <strong>result-cursor-columns:</strong><br>
	 *  	<ul>
	 * 		<li>"message" - Result-Message. Empty: nothing is displayed to the user, "ok": success-Toast is displayed. Everything else results in a dialog to be acknowleged.</li>
	 *  	</ul>
	 *  </li>
	 *  
	 * 
	 * <li><strong>content://&lt;your package&gt;/downloadMultiple</strong><br><br>
	 *  Download multiple caches, either a comma seperated list of gc-codes or an area defined
	 *  by coordinates and a radius. (Currently GeOrg only supports radius-requests)
	 *  <strong>input-parameters: </strong>
	 *   <ol>
	 *  	<li>The type of the request, either "radius" or "list"</li>
	 *  	<li>Depending on the type of the request the number of parameters varies:
	 *        <ul>
	 *          <li>radius:
	 *          <ul>
	 *           <li>latitude as a double</li>
	 *           <li>longitude as a double</li>
	 *           <li>radius in meters</li>
	 *          </ul></li>
	 *          <li>list
	 *           <ul>
	 *           <li>a string of comma-separated GC-Codes for each cache to download</li>
	 *          </ul></li>
	 *        </ul>
	 *      </li>
	 *  </ol>
	 *  <strong>result-cursor-columns:</strong><br>
	 *  	<ul>
	 * 		<li>"file"    - On-device file-path to the resulting GPX-file. Make sure GeOrg can access this
	 *                      file.</li>
	 * 		<li>"message" - Result-Message in case of error or empty.</li>
	 *  	</ul>
	 *  </li>
	 *  <li><strong>content://&lt;your package&gt;/syncCaches</strong><br><br>
	 *  Update all previously downloaded caches that have been changed since the last download.
	 *  <strong>result-cursor-columns:</strong><br>
	 *  	<ul>
	 * 		<li>"file"    - On-device file-path to the resulting GPX-file. Make sure GeOrg can access this
	 *                      file.</li>
	 * 		<li>"message" - Result-Message in case of error or empty.</li>
	 *  	</ul>
	 *  </li>
	 * </ul>
	 * </ul>
 	 * Errors are always given in the column "message". If it's not null this will be 
	 * interpreted as an error in the frontend, exception for push-methods, where the
	 * message contains either an error or "ok" if all went well.
	 *
	 * @see android.content.ContentProvider#query(android.net.Uri, java.lang.String[], java.lang.String, java.lang.String[], java.lang.String)
	 */
	@Override
	public Cursor query(Uri aUri, String[] aProjection, String aSelection,
			String[] aSelectionArgs, String aSortOrder) {
		
		if (aUri.toString().endsWith(ConnectorSupportedQueries.LIST_CACHES_BY_COORD)) {
			String lvFilter = "found=fdn";
			if (aSelectionArgs.length>4) {
				lvFilter = aSelectionArgs[4];
			}
			boolean lvShowAddWPs = false;
			if (aSelectionArgs.length>5) {
				lvShowAddWPs = Boolean.parseBoolean(aSelectionArgs[5]);
			}
			return ivLoader.readCachesByCoord(
					Double.valueOf(aSelectionArgs[0])/1E6, 
					Double.valueOf(aSelectionArgs[1])/1E6,
					Double.valueOf(aSelectionArgs[2])/1E6, 
					Double.valueOf(aSelectionArgs[3])/1E6,
					lvFilter,
					lvShowAddWPs);
			
		} else if (aUri.toString().endsWith(ConnectorSupportedQueries.GET_CACHE_DETAILS)) {
			return ivLoader.readCacheDetails(
					aSelectionArgs[0]);      
//			aSelectionArgs[0],       //GCCode
//			aSelectionArgs[0]);      //ID
		} else if (aUri.toString().endsWith(ConnectorSupportedQueries.GET_CACHE_GPX)) {
			String lvSpoilerDir = null;
			if (aSelectionArgs.length > 1) {
				lvSpoilerDir = aSelectionArgs[1];
			}
			return ivLoader.readGPX(aSelectionArgs[0], lvSpoilerDir);
		} else if (aUri.toString().endsWith(ConnectorSupportedQueries.PUSH_LOG)) {
			return ivLoader.pushLog(aSelectionArgs[0], aSelectionArgs[1], aSelectionArgs[2], aSelectionArgs[3], aSelectionArgs[4], aSelectionArgs[5]);
		} else if (aUri.toString().endsWith(ConnectorSupportedQueries.PUSH_FIELD_NOTES)) {
			return ivLoader.pushFieldNotes(aSelectionArgs[0]);
			
		} else if (aUri.toString().endsWith(ConnectorSupportedQueries.PING)) {
			MatrixCursor lvPingResult = new MatrixCursor(new String[]{"result"});
			lvPingResult.addRow(new String[]{"OK"});
			return lvPingResult;
		} else if (aUri.toString().endsWith(ConnectorSupportedQueries.GET_SUPPORTED_QUERIES)) {
			return ivLoader.getSupportedQueries();
		} else if (aUri.toString().endsWith(ConnectorSupportedQueries.GET_COMMANDS)) {
			return ivLoader.getCommands();
		} else if (aUri.toString().endsWith(ConnectorSupportedQueries.GET_SPOILERS)) {
			return ivLoader.readSpoilers(aSelectionArgs[0], aSelectionArgs[1]);
		} else if (aUri.toString().endsWith(ConnectorSupportedQueries.EXECUTE_COMMAND)) {
			return ivLoader.executeCommand(aSelectionArgs[0]);
		} else if (aUri.toString().endsWith(ConnectorSupportedQueries.DOWNLOAD_MULTIPLE)) {
			return ivLoader.downloadMultiple(aSelectionArgs);
		} else if (aUri.toString().endsWith(ConnectorSupportedQueries.SYNC_CACHES)) {
			return ivLoader.syncCaches();
		}
		
		return null;
	}
	
	/**
	 * We don't delete nothing ...
	 * @see android.content.ContentProvider#delete(android.net.Uri, java.lang.String, java.lang.String[])
	 */
	@Override
	public int delete(Uri aUri, String aSelection, String[] aSelectionArgs) {
		return 0;
	}



	/**
	 * Won't insert anything either ...
	 * @see android.content.ContentProvider#insert(android.net.Uri, android.content.ContentValues)
	 */
	@Override
	public Uri insert(Uri aUri, ContentValues aValues) {
		// Nothing to insert
		return null;
	}
	
	/**
	 * Updatezn't ...
	 * @see android.content.ContentProvider#update(android.net.Uri, android.content.ContentValues, java.lang.String, java.lang.String[])
	 */
	@Override
	public int update(Uri aUri, ContentValues aValues, String aSelection,
			String[] aSelectionArgs) {
		return 0;
	}

	/**
	 * Not needed
	 * 
	 * @see android.content.ContentProvider#getType(android.net.Uri)
	 */
	@Override
	public String getType(Uri aUri) {
		return null;
	}
}
